/* $Id: gc.h,v 1.4 2006/01/21 16:17:14 cegger Exp $
***************************************************************************

   X server for LibGGI - GC (graphics context) acceleration

   Copyright (C) 1997      Michael Krause	[rawstyle@ms.demo.org]
   Copyright (C) 1998-2000 Marcus Sundberg	[marcus@ggi-project.org]
   Copyright (C) 2003-2006 Christoph Egger

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

***************************************************************************
*/

#ifndef _XGGI_ACCEL_GC_H
#define _XGGI_ACCEL_GC_H


#define PSZ 8
#include <ggi/gg.h>

#include "mi.h"
#include "cfb.h"
#include "cfb16.h"
#include "cfb24.h"
#include "cfb32.h"

#include "scrnintstr.h"
#include "gcstruct.h"

#include "xggi.h"
#include "xggi_misc.h"
#include "xggi_struct.h"


extern GCOps xggiGCOps;


#define XGGI_GC_OP_PROLOGUE(pGC, pDrawable)		\
	int scr = xggiGetScreenIdx(pGC->pScreen);	\
	xggiGCPrivPtr pGCPriv = GCBACKUP(scr, pGC);	\
	GCFuncs *oldFuncs = pGC->funcs;			\
	unwrap(pGCPriv, pGC, funcs);			\
	unwrap(pGCPriv, pGC, ops);

#define XGGI_GC_OP_EPILOGUE(pGC, pDrawable)		\
	wrap (pGCPriv, pGC, funcs, oldFuncs);		\
	wrap (pGCPriv, pGC, ops, &xggiGCOps);


/* implements pScreen->CreateGC */
Bool xggiCreateGC(GCPtr pGC);


/* fillspans.c */
void xggiFillSpans(
		DrawablePtr pDrawable,
		GCPtr pGC,
		int nInit,
		DDXPointPtr pptInit,
		int *pwidthInit,
		int fSorted);

/* setspans.c */
void xggiSetSpans(
		DrawablePtr pDrawable,
		GCPtr pGC,
		char *psrc,
		DDXPointPtr ppt,
		int *pwidth,
		int nspans,
		int fSorted);

/* putimage.c */
void xggiPutImage(
		DrawablePtr pDrawable,
		GCPtr pGC, int depth,
		int x, int y,
		int w, int h,
		int leftPad, int format,
		char *pBits);

/* copyarea.c */
RegionPtr xggiCopyArea(
		DrawablePtr pSrcDrawable,
		DrawablePtr pDstDrawable,
		GCPtr pGC,
		int srcx, int srcy,
		int width, int height,
		int dstx, int dsty);

/* copyplane.c */
RegionPtr xggiCopyPlane(
		DrawablePtr pSrcDrawable,
		DrawablePtr pDstDrawable,
		GCPtr pGC,
		int srcx, int srcy,
		int width, int height,
		int dstx, int dsty,
		unsigned long bitPlane);

/* polypoint.c */
void xggiPolyPoint(
		DrawablePtr pDrawable,
		GCPtr pGC,
		int mode,
		int npt,
		DDXPointPtr pptInit);

/* polylines.c */
void xggiPolylines(
		DrawablePtr pDrawable,
		GCPtr pGC,
		int mode,
		int npt,
		DDXPointPtr pptInit);

/* polysegment.c */
void xggiPolySegment(
		DrawablePtr pDrawable,
		GCPtr pGC,
		int nseg,
		xSegment *pSegs);

/* polyrectangle.c */
void xggiPolyRectangle(
		DrawablePtr pDrawable,
		GCPtr pGC,
		int nrects,
		xRectangle *pRects);

/* polyarc.c */
void xggiPolyArc(
		DrawablePtr pDrawable,
		GCPtr pGC,
		int narcs,
		xArc *parcs);

/* fillpolygon.c */
void xggiFillPolygon(
		DrawablePtr pDrawable,
		GCPtr pGC,
		int shape,
		int mode,
		int count,
		DDXPointPtr pPts);

/* polyfillrect.c */
void xggiPolyFillRect(
		DrawablePtr pDrawable,
		GCPtr pGC,
		int nrectFill,
		xRectangle *prectInit);

/* polyfillarc.c */
void xggiPolyFillArc(
		DrawablePtr pDrawable,
		GCPtr pGC,
		int narcs,
		xArc *parcs);

/* polytext8.c */
int xggiPolyText8(
		DrawablePtr pDrawable,
		GCPtr pGC,
		int x, int y,
		int count,
		char *chars);

/* polytext16.c */
int xggiPolyText16(
		DrawablePtr pDrawable,
		GCPtr pGC,
		int x, int y,
		int count,
		unsigned short *chars);

/* imagetext8.c */
void xggiImageText8(
		DrawablePtr pDrawable,
		GCPtr pGC,
		int x, int y,
		int count,
		char *chars);

/* imagetext16.c */
void xggiImageText16(
		DrawablePtr pDrawable,
		GCPtr pGC,
		int x, int y,
		int count,
		unsigned short *chars);

/* imageglyphblt.c */
void xggiImageGlyphBlt(
		DrawablePtr pDrawable,
		GCPtr pGC,
		int x, int y,
		unsigned int nglyph,
		CharInfoPtr *ppci,
		pointer pglyphBase);

/* polyglyphblt.c */
void xggiPolyGlyphBlt(
		DrawablePtr pDrawable,
		GCPtr pGC,
		int x, int y,
		unsigned int nglyph,
		CharInfoPtr *ppci,
		pointer pglyphBase);

/* pushpixels.c */
void xggiPushPixels(
		GCPtr pGC,
		PixmapPtr pBitMap,
		DrawablePtr pDst,
		int w, int h,
		int x, int y);



#endif	/* _XGGI_ACCEL_GC_H */
